@@ -47,6 +47,7 @@ module Yesod.Core.Handler
     , fileName
     , fileContentType
     , fileSource
+    , fileSourceByteString
     , fileMove
       -- *** Convenience functions
     , languages
@@ -118,6 +119,7 @@ module Yesod.Core.Handler
     , setHeader
     , replaceOrAddHeader
     , setLanguage
+    , addContentDispositionFileName
       -- ** Content caching and expiration
     , cacheSeconds
     , neverExpires
@@ -148,6 +150,7 @@ module Yesod.Core.Handler
     , setMessageI
     , getMessage
       -- * Subsites
+    , SubHandlerFor
     , getSubYesod
     , getRouteToParent
     , getSubCurrentRoute
@@ -251,6 +254,7 @@ import           Data.CaseInsensitive (CI, original)
 import qualified Data.Conduit.List as CL
 import           Control.Monad.Trans.Resource  (MonadResource, InternalState, runResourceT, withInternalState, getInternalState, liftResourceT, resourceForkIO)
 import qualified System.PosixCompat.Files as PC
+import           Conduit ((.|), runConduit, sinkLazy)
 import           Data.Conduit (ConduitT, transPipe, Flush (Flush), yield, Void)
 import qualified Yesod.Core.TypeCache as Cache
 import qualified Data.Word8 as W8
@@ -779,6 +783,26 @@ deleteCookie a = addHeaderInternal . DeleteCookie (encodeUtf8 a) . encodeUtf8
 setLanguage :: MonadHandler m => Text -> m ()
 setLanguage = setSession langKey
 
+-- | Set attachment file name.
+--
+-- Allows Unicode characters by encoding to UTF-8.
+-- Some modurn browser parse UTF-8 characters with out encoding setting.
+-- But, for example IE9 can't parse UTF-8 characters.
+-- This function use
+-- <https://tools.ietf.org/html/rfc6266 RFC 6266>(<https://tools.ietf.org/html/rfc5987 RFC 5987>)
+--
+-- @since 1.6.4
+addContentDispositionFileName :: MonadHandler m => T.Text -> m ()
+addContentDispositionFileName fileName
+    = addHeader "Content-Disposition" $ rfc6266Utf8FileName fileName
+
+-- | <https://tools.ietf.org/html/rfc6266 RFC 6266> Unicode attachment filename.
+--
+-- > rfc6266Utf8FileName (Data.Text.pack "€")
+-- "attachment; filename*=UTF-8''%E2%82%AC"
+rfc6266Utf8FileName :: T.Text -> T.Text
+rfc6266Utf8FileName fileName = "attachment; filename*=UTF-8''" `mappend` decodeUtf8 (H.urlEncode True (encodeUtf8 fileName))
+
 -- | Set an arbitrary response header.
 --
 -- Note that, while the data type used here is 'Text', you must provide only
@@ -1359,6 +1383,17 @@ rawRequestBody = do
 fileSource :: MonadResource m => FileInfo -> ConduitT () S.ByteString m ()
 fileSource = transPipe liftResourceT . fileSourceRaw
 
+-- | Extract a strict `ByteString` body from a `FileInfo`.
+--
+-- This function will block while reading the file.
+--
+-- > do
+-- >     fileByteString <- fileSourceByteString fileInfo
+--
+-- @since 1.6.5
+fileSourceByteString :: MonadResource m => FileInfo -> m S.ByteString
+fileSourceByteString fileInfo = runConduit (L.toStrict <$> (fileSource fileInfo .| sinkLazy))
+
 -- | Provide a pure value for the response body.
 --
 -- > respond ct = return . TypedContent ct . toContent
