@@ -1,7 +1,7 @@
 {-# LANGUAGE CPP #-}
 -- | This is designed to be used as
 --
--- > qualified import Yesod.Core.Unsafe as Unsafe
+-- > import qualified Yesod.Core.Unsafe as Unsafe
 --
 -- This serves as a reminder that the functions are unsafe to use in many situations.
 module Yesod.Core.Unsafe (runFakeHandler, fakeHandlerGetLogger) where
@@ -19,7 +19,10 @@ import Control.Monad.IO.Class (MonadIO)
 --
 -- > unsafeHandler = Unsafe.fakeHandlerGetLogger appLogger
 fakeHandlerGetLogger :: (Yesod site, MonadIO m)
-                     => (site -> Logger) -> site -> HandlerT site IO a -> m a
+                     => (site -> Logger)
+                     -> site
+                     -> HandlerFor site a
+                     -> m a
 fakeHandlerGetLogger getLogger app f =
      runFakeHandler mempty getLogger app f
  >>= either (error . ("runFakeHandler issue: " `mappend`) . show)
